/******************************************************************************
*******************************************************************************
****	Copyright (c) 2001
****	Conexant Systems Inc. (formerly Rockwell Semiconductor Systems)
****	Personal Computing Division
****	All Rights Reserved
****
****	CONFIDENTIAL and PROPRIETARY --
****		No Dissemination or use without prior written permission.
****
*******************************************************************************
**
**MODULE NAME:
**	????
**
**FILE NAME:
**	cnxadslautolog.c
**
**ABSTRACT:
**	This file implements the auto log application for the Tigris ADSL
**	device.
**
**DETAILS:
**	????
**
*******************************************************************************
** KEYWORDS:
**	$Archive: /Projects/Tigris and Derivatives Linux/Reference Designs/Yukon-AV/Linux_2.4_ATM_Driver/DownLoadApp/cnxadslautolog.c $
**	$Revision: 1 $
**	$Date: 7/05/01 4:45p $
*******************************************************************************
******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <ctype.h>
#include <getopt.h>
#include <wait.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <stdint.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/time.h>

#include <atm.h>

#include "../LnxDefs.h"
#include "../CommonData.h"
#include "../ConfigLabels.h"

#if 1
#include "../KernelModule/Product.h"
#include "../KernelModule/DevIo.h"
#else
#include "atm_cnxadsl.h"
#endif

#include "readuserparams.h"

#define SOCKET_LEN			256
#define POLL_INTERVAL_1_SEC		1
#define SHOWTIME_MSG			"Line connected.\n"
#define OPEN_WRITEONLY_EMPTY		(O_WRONLY | O_TRUNC | O_CREAT)
#define OPEN_WRITEONLY			(O_WRONLY | O_CREAT)
#define OPEN_WRITEONLY_APPEND		(O_WRONLY | O_APPEND)
#define FILE_PERMISSIONS		(S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP)

// local functions

static NTSTATUS SendIoctlCmd(
			        IN int			SocketId,
				IN int			CommandId,
				IN struct atmif_sioc	*pSocketioc);

static NTSTATUS ReadUserParams(
			        IN  const char *	csPath,
				OUT unsigned long *	pAutoLogEnable,
				OUT char *		csAutoLogPath,
				OUT char *		csFirmwareFilename,
				OUT unsigned long *	pAutoLogFrequency,
				OUT unsigned long *	pAutoLogMaxSize,
				OUT unsigned long *	pAutoLogOverWrite);

static void* DoAutoLogPoll(
			        IN unsigned long	AutoLogEnable,
				IN char *		csAutoLogFile,
				IN unsigned long	AutoLogFrequency,
				IN unsigned long	AutoLogMaxSize,
				IN unsigned long	AutoOverwrite);

static struct atmif_sioc	Socketioc;

/******************************************************************************
FUNCTION NAME:
	main

ABSTRACT:
	This is main entry point.

PARAMETERS:
	argc -	Command line parameter count
	argv -	Command line parameters

RETURN:
	The status of the operation.

DETAILS:
******************************************************************************/
int main(int argc,char **argv)
{
  NTSTATUS		NTStatus;
  char			csFileNameFromConf[512];
  char			csAutoLogFile[512];
  unsigned long		AutoLogEnable;
  unsigned long		AutoLogFrequency;
  unsigned long		AutoLogMaxSize;
  unsigned long		AutoLogOverwrite;
  int			c;
  int			h_opt = 0;
  int			opt_err = 0;

  while ((c = getopt(argc,argv,"hi:")) != EOF) {
    switch (c) {
    case 'h' :
      h_opt |= 1;
      opt_err |= 1;
      break;
    case 'i' :
      	Socketioc.number = atoi(optarg);
      break;
    default :
      opt_err |= 1;
      break;
    }
  }

  if (opt_err || ((argc - optind) != 1)) {
    printf("Usage: cnxadslautolog [-h][-i itf] Path\n");
    if (h_opt)
      printf("-h     help\n"
	     "-i itf atm interface number\n");
    exit(1);
  }

  // read the user configurable parameters from the driver configuration
  // file to get the autologging parameters.
  NTStatus = ReadUserParams(argv[optind],
			    &AutoLogEnable,
			    csAutoLogFile,
			    csFileNameFromConf,
			    &AutoLogFrequency,
			    &AutoLogMaxSize,
			    &AutoLogOverwrite);
  if (!NT_SUCCESS(NTStatus))
  {
    printf("***ERROR - read user parameters (ReadUserParams) failed, NTStatus=%lx\n", NTStatus);
    return STATUS_FAILURE;
  }

  DoAutoLogPoll(AutoLogEnable,
  		csAutoLogFile,
  		AutoLogFrequency,
  		AutoLogMaxSize,
  		AutoLogOverwrite);

  return STATUS_SUCCESS;
}

/******************************************************************************
FUNCTION NAME:
	DoAutoLogPoll

ABSTRACT:
	This function will open the socket and make calls to the driver to
	implement the automatic logging.

PARAMETERS:
	AutoLogEnable -		Flag enabling the logging function
	csAutoLogFile -		File to receive log data
	AutoLogFrequency -	Frequency to perform logging
	AutoLogMaxSize -	Maximum allowed size of the log file
	AutoLogOverwrite -	Clear the log file before starting

RETURN:
	None

DETAILS:
	This function periodically issues an IOCTL call to the driver to
	execute automatic logging of the ADSL driver events.
	The polling of the driver will continue until the one of the IOCTL
	commands fail, which probably indicates that the driver has been
	unloaded.
******************************************************************************/
void* DoAutoLogPoll(
    IN unsigned long		AutoLogEnable,
    IN char *			csAutoLogFile,
    IN unsigned long		AutoLogFrequency,
    IN unsigned long		AutoLogMaxSize,
    IN unsigned long		AutoLogOverwrite)
{
  int				Status;
  long int			LogStat;
  int				SocketId;
  /* See DevIoCardAL.h
  BD_DRIVER_LOG_WITH_TIME_T	Buffer;
  */ //Buffer[3]==Buffer.DwordArray
  DWORD				Buffer[SOCKET_LEN];
  int				Length;
  int				FilePtr;
  TIG_DEVICE_SPEC_T		DevSpecific;
  BACK_DOOR_T *			pBackDoorBuf;
  ULONG				ShowTimeFlag;
  int				FileLength;

  pBackDoorBuf = (BACK_DOOR_T*) &DevSpecific.BackDoorBuf;
  Status = STATUS_SUCCESS;
  FileLength = 0;

  if (AutoLogEnable)
  {
    DWORD		OpenFlags;

    // if overwriting then clear the file first
    OpenFlags = AutoLogOverwrite ? OPEN_WRITEONLY_EMPTY : OPEN_WRITEONLY;

    FilePtr = open(csAutoLogFile, OpenFlags, FILE_PERMISSIONS);

    if (FilePtr >= 0)
      close(FilePtr);
    else
    {
      printf("***ERROR - Failed to open AutoLog file!\n ");
      printf("\nAutoLog NOT started!\n ");
      return NULL;
    }
  }
  else
  {
    printf("AutoLog is not enabled in the driver configuration file.\n ");
    printf("\nAutoLog NOT started!\n ");
    return NULL;
  }

  // open the socket to the device.  This socket
  // will be used to issue the load commands
  SocketId = socket(PF_ATMPVC, SOCK_DGRAM, 0);
  if (SocketId < 0)
  {
    printf("Socket open failure.\n");
    printf("\nAutoLog NOT started!\n");
    return NULL;
  }

  // periodically poll the driver
  while (Status == STATUS_SUCCESS)
  {
    // the poll should be right at one second.
    // But it's not really that critical because
    // the driver actually uses the realtime clock value for timing
    sleep(POLL_INTERVAL_1_SEC);

    // open the log file
    FilePtr = open(csAutoLogFile, OPEN_WRITEONLY_APPEND);

    if (FilePtr >= 0) {
      FileLength = lseek(FilePtr, 0, SEEK_END);

      if (!AutoLogMaxSize || (FileLength < AutoLogMaxSize))
      {
	// initialize the length
	Buffer[0] = 0; //Buffer.FirstDword.Size=0;

	Socketioc.arg = &Buffer;
	Socketioc.length = SOCKET_LEN;

	// call the function to get the log data
	LogStat = SendIoctlCmd(SocketId, TIG_GET_LOG, &Socketioc);
	if (LogStat != STATUS_SUCCESS)
	  break;

	Length = Buffer[0] >> 16;//Buffer.FirstDword.Size

	// if there is anything to log then log it
	if ((LogStat == STATUS_SUCCESS) && (Length != 0))
	  write(FilePtr, &Buffer[3], strlen((char*) &Buffer[3]));

	Socketioc.arg = &ShowTimeFlag;
	Socketioc.length = sizeof(ShowTimeFlag);

	// now log some additional statistics, get the showtime status
	LogStat = SendIoctlCmd(SocketId, TIG_IS_SHOWTIME, &Socketioc);
	if (LogStat != STATUS_SUCCESS)
	{
	  printf("The AutoLog IOCTL command to the driver failed!\n");
	  break;
	}

	// if not in showtime - log not connected
	if (!ShowTimeFlag) {
	  static const char csNoShowTimeMsg[] = "Line not connected!\n";

	  write(FilePtr, csNoShowTimeMsg, strlen(csNoShowTimeMsg));
	}
	else
	{
	  char  		       PrintBuf[512];

	  // get the stats

	  memset(pBackDoorBuf, 0, sizeof(BACK_DOOR_T));

	  pBackDoorBuf->TotalSize = sizeof(BACK_DOOR_T);
	  pBackDoorBuf->ReqCode = BD_FRAMEAL_ATM_GET_STATS;

	  pBackDoorBuf->Params.BdFrameALAtmStats.VcIndex = BD_ATM_STATS_INDEX_FOR_LINK;

	  Socketioc.arg = &DevSpecific;
	  Socketioc.length = sizeof(DevSpecific);

	  LogStat = SendIoctlCmd(SocketId, TIG_DEVICE_SPEC, &Socketioc);
	  if (LogStat != STATUS_SUCCESS)
	  {
	    printf("The AutoLog IOCTL command to the driver failed!\n");
	    break;
	  }

	  sprintf(PrintBuf, "Cells Received: %ld   Transmitted: %ld\n",
		  pBackDoorBuf->Params.BdFrameALAtmStats.NumRxCells,
		  pBackDoorBuf->Params.BdFrameALAtmStats.NumTxCells);

	  write(FilePtr, PrintBuf, strlen(PrintBuf));
	}
      }

      // close the file
      close(FilePtr);
    }
  }

  printf("\nAutoLog Terminated!\n");

  // clean up after yourself before leaving
  close(SocketId);

  return NULL;
}

NTSTATUS ReadUserParams(
    IN  const char *	csPath,
    OUT unsigned long *	pAutoLogEnable,
    OUT char *		csAutoLogPath,
    OUT char *		csFirmwareFilename,
    OUT unsigned long *	pAutoLogFrequency,
    OUT unsigned long *	pAutoLogMaxSize,
    OUT unsigned long *	pAutoLogOverwrite)
{
  TIG_USER_PARAMS	TigParams;

  return ReadUserParamsE(csPath,
			 pAutoLogEnable,
			 csAutoLogPath,
			 csFirmwareFilename,
			 pAutoLogFrequency,
			 pAutoLogMaxSize,
			 pAutoLogOverwrite,
			 &TigParams);
}

/******************************************************************************
FUNCTION NAME:
	SendIoctlCmd

ABSTRACT:
	This function is called to send an IOCTL to the device driver.

PARAMETERS:
	SocketId	- identifies the socket to send to
	CommandId	- 
	cInRec		- pointer to the record data to send
	RecordLength	- length of the record in bytes

RETURN:
	The status of the operation.

DETAILS:
******************************************************************************/
NTSTATUS SendIoctlCmd(
    IN int			SocketId,
    IN int			CommandId,
    IN struct atmif_sioc	*pSocketioc)
{
  int			Status;

  Status = ioctl(SocketId, CommandId, &Socketioc);
  if (Status < 0)
  {
    printf("The AutoLog IOCTL command (0x%x) to the driver failed (Status=0x%x)!\n", CommandId, Status);
    return STATUS_FAILURE;
  } else
    return STATUS_SUCCESS;
}
