#include <stdio.h>

#define DWORD unsigned long
#define WORD unsigned short
#define BYTE unsigned char

#define POLYNOMIAL 0x0633

#define HEC_GENERATOR   0x107               /* x^8 + x^2 +  x  + 1  */
#define COSET_LEADER    0x055               /* x^6 + x^4 + x^2 + 1  */

#define NO_ERROR_DETECTED   0xff
#define UNCORRECTIBLE_ERROR 0x80

static unsigned char err_posn_table[256];
static unsigned char syndrome_table[256];

#define PRBS_2_23_1_POLYNOMIAL  0x00420000UL    /* x^23 + x^18 + 1 */
#define PBRS_2_20_1_POLYNOMIAL	0x00090000UL	/* x^20+ x^17 + 1 */
#define PRBS_2_15_1_POLYNOMIAL  0x00006000UL    /* x^15 + x^14 + 1 */

static const struct {unsigned long v; char *name;} poly[] = {
  {PRBS_2_23_1_POLYNOMIAL,"PRBS_2_23_1_POLYNOMIAL"},
  {PBRS_2_20_1_POLYNOMIAL,"PBRS_2_20_1_POLYNOMIAL"},
  {PRBS_2_15_1_POLYNOMIAL,"PRBS_2_15_1_POLYNOMIAL"}
};

typedef unsigned char row256[256];
typedef row256 PRBS_ARRAY[4];

/* table of weights for 8-bit bytes */
static row256 Weight;
static PRBS_ARRAY PRBSTable;

static row256 ReverseByteTable;

static WORD Crc10Table[256];

/* generate the table of CRC-10 remainders for all possible bytes */
static void FrameALOAMGenCRC10Table(void) {
  DWORD RowIndex;

  for ( RowIndex = 0; RowIndex < 256; RowIndex++ ) {
    DWORD BitIndex;
    WORD Crc10Accum;

    Crc10Accum = (unsigned short) RowIndex << 2;
    for ( BitIndex = 0; BitIndex < 8; BitIndex++ )
      if ((Crc10Accum <<= 1) & 0x400)
	Crc10Accum ^= POLYNOMIAL;

    Crc10Table[RowIndex] = Crc10Accum;
  }
}

static void genRev(void) {
  DWORD Index;

// Reverse Bit Ordering of a Byte
#define REVERSE_BYTE(n) \
			((n & 0x80) >> 7) | \
			((n & 0x40) >> 5) | \
			((n & 0x20) >> 3) | \
			((n & 0x10) >> 1) | \
			((n & 0x08) << 1) | \
			((n & 0x04) << 3) | \
			((n & 0x02) << 5) | \
			((n & 0x01) << 7)

  for (Index = 0; Index < 256; Index++ )
    ReverseByteTable[Index] = REVERSE_BYTE(Index);

#undef REVERSE_BYTE
}

static unsigned long ParityCheck (unsigned long v) {
  unsigned long Parity=0UL;
  int i;

  for (i = 0; v && (i < 32); i++) {
    Parity ^= v;
    v >>= 1;
  }
  return Parity & 1;
}

static void genPRBSTable(unsigned long Polynomial) {
  DWORD  Index, RowIndex, BitIndex;

  // generate table for 32 PRBS
  for (Index = 0; Index < 4; Index++)
    for (RowIndex = 0; RowIndex < 256; RowIndex++) {
      unsigned long prbs_accum = ((unsigned long)RowIndex << (Index * 8));

      for (BitIndex = 0; BitIndex < 8; BitIndex++)
	prbs_accum = (prbs_accum << 1) ^ ParityCheck(prbs_accum & Polynomial);
      PRBSTable[Index][RowIndex] = (prbs_accum & 0xff);
    }
}

static void genWeight(void) {
  DWORD Index, BitIndex;

  for (Index = 0; Index < 256; Index++) {
    unsigned char mask = Index, ones_count = 0;

    for (BitIndex = 0; mask && (BitIndex < 8); BitIndex++) {
      if (mask & 1)
	ones_count++;
      mask >>= 1;
    }
    Weight[Index] = ones_count;
  }
}

/* generate a table of CRC-8 syndromes for all possible input bytes */
static void FrameALHECInit(void) {
  WORD i;
  BYTE hec_accum;

  for ( i = 0; i < 256; i++ ) {
    WORD j, syndrome;

    syndrome = i;
    for ( j = 0; j < 8; j++ )
      if ( syndrome & 0x80 )
	syndrome = (syndrome << 1) ^ HEC_GENERATOR;
      else
	syndrome = (syndrome << 1);

    syndrome_table[i] = (unsigned char) syndrome;
  }

  /* Generate the error position table as a function of the syndrome. */
  /* A negative value indicates that there is no error to correct.    */
  /* A value of 40 or greater indicates that an uncorrectible error   */
  /* has occurred, i.e., that the syndrome either indicates a double  */
  /* error pattern or points to a single-bit error position that is   */
  /* outside the header.  Finally, a value between 0 and 39 indicates */
  /* that a single-bit error occurred in that position (bit 0 being   */
  /* the header bit which was transmitted first).                     */

  /* mark the NO_ERROR_DETECTED position */
  err_posn_table[0] = NO_ERROR_DETECTED;

  /* default remaining positions to UNCORRECTIBLE */
  for ( i = 1; i < 256; i++ )
    err_posn_table[i] = UNCORRECTIBLE_ERROR;

  /* override w/err posn for syndromes indicating other single-bit errors */
  for ( i = 0; i < 5; i++ ) {
    WORD j;

    for ( j = 0; j < 8; j++ ) {
      if ( i < 4) {
	WORD k;

	hec_accum = 0;
	for ( k = 0; k < 4; k++ )
	  if ( k == i)
	    hec_accum = syndrome_table [ hec_accum ^ (0x80 >> j) ];
	  else
	    hec_accum = syndrome_table [ hec_accum ];
      } else
	hec_accum = (0x80 >> j);

      err_posn_table [ hec_accum ] = (i * 8) + j;
    }
  }
}

static void dumpB256(const BYTE *tab) {
  int r, c;

  for (r = 0; r < 256/16; r++) {
    for (c = 0; c < 16; c++)
      printf("0x%02x,", tab[r*16+c]);
    printf("\n");
  }
}

static void dumpW256(const WORD *tab) {
  int r, c;

  for (r = 0; r < 256/16; r++) {
    for (c = 0; c < 16; c++)
      printf("0x%04x,", tab[r*16+c]);
    printf("\n");
  }
}

static void dumpB256T(const char *name, const BYTE *tab) {
  printf("static const BYTE %s[256] = {\n", name);

  dumpB256(tab);

  printf("};\n");
}

static void dumpW256T(const char *name, const WORD *tab) {
  printf("static const WORD %s[256] = {\n", name);

  dumpW256(tab);

  printf("};\n");
}

int main(void) {
  int i;

  genRev();

  dumpB256T("ReverseByteTable", ReverseByteTable);

  genWeight();

  dumpB256T("Weight", Weight);

  for (i=0; i < sizeof(poly)/sizeof(poly[0]); i++) {
    int j;

    genPRBSTable(poly[i].v);

    printf("static const PRBS_ARRAY PRBSTable_%s = {\n", poly[i].name);

    for (j = 0; j < 4; j++)
      dumpB256(PRBSTable[j]);

    printf("};\n");
  }

  FrameALHECInit();

  dumpB256T("err_posn_table", err_posn_table);
  dumpB256T("syndrome_table", syndrome_table);

  FrameALOAMGenCRC10Table();

  dumpW256T("Crc10Table", Crc10Table);

  return 0;
}
